/* Copyright (c) 1988 Carrick Sean Casey. All rights reserved. */

/* accept and initialize a new client connection */

/* returns: */
/* the resultant socket file descriptor */
/*  0 if operation would block */
/* -1 if operation failed */

#include <stdio.h>
#include "config.h"
#ifdef FCNTL
#include <fcntl.h>
#endif				/* FCNTL */
#ifdef SOLARIS
#include <sys/file.h>
#endif				/* SOLARIS */
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>
#include "functions.h"
#include "externs.h"
#if defined(hpux) || defined(_SEQUENT_)
#include <sys/file.h>
#endif				/* hpux || _SEQUENT_ */
#ifdef linux
#include "select.h"
#endif

#ifdef SOLARIS
extern char *sys_errlist[];
#endif				/* SOLARIS */
/* extern int errno; */

int
_newconnect(s)
	int     s;
{
	int     ns;
	int     fromlen;
	int     one = 1;
	struct sockaddr from;
	int     flags;
	struct linger nolinger;
	char	tbuf[256];

	/* accept the connection */
	fromlen = sizeof(from);
	if ((ns = accept(s, (struct sockaddr *) & from, &fromlen)) < 0)
		if (errno == EWOULDBLOCK)
			return (0);
		else {
			perror("new_client:accept");
			return (-1);
		}

	/* force occasional connection check */
	if (setsockopt(ns, SOL_SOCKET, SO_KEEPALIVE,
		(char *) &one, sizeof(one)) < 0) {
		perror("newclient:setsockopt (keepalive)");
		sprintf(tbuf, "newconnect:setsockopt (keepalive): %s",
			sys_errlist[errno]);
		mdb(tbuf);
	}
	/* don't allow broken connections to linger */
	nolinger.l_onoff = 1;
	nolinger.l_linger = 0;
	if (setsockopt(ns, SOL_SOCKET, SO_LINGER,
		(char *) &nolinger, sizeof(nolinger)) < 0) {
		perror("newclient:setsockopt (dontlinger)");
		sprintf(tbuf, "newconnect:setsockopt (dontlinger) %s",
			sys_errlist[errno]);
		mdb(tbuf);
	}
	one = 24576;
	if (setsockopt(s, SOL_SOCKET, SO_SNDBUF,
		(char *) &one, sizeof(one)) < 0) {
		perror("SO_SNDBUF");
		sprintf(tbuf, "newconnect.c:SO_SNDBUF %s",
			sys_errlist[errno]);
		mdb(tbuf);
	}
	/* make the socket non-blocking */
	if (fcntl(ns, F_SETFL, FNDELAY) < 0) {
		perror("new_client:fcntl");
		sprintf(tbuf, "newconnect:fcntl %s",
			sys_errlist[errno]);
		mdb(tbuf);
		return (-1);
	}
	/* Don't close on exec */
	flags = fcntl(ns, F_GETFD, 0);
	flags = flags & ~FD_CLOEXEC;
	fcntl(ns, F_SETFD, flags);

	/* enter the user's fd into the active set */
	FD_SET(ns, &fdset);

	if (ns > highestfd)
		highestfd = ns;

	/* first command is a "new" command */
	cbufs[ns].new = 1;

	return (ns);
}
