/* Copyright (c) 1988 Carrick Sean Casey. All rights reserved. */

/* main program loop */
/* active clients, new connections, and perform asynch dungeon action */

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#ifdef FCNTL
#include <fcntl.h>
#endif				/* FCNTL */
#ifdef STRINGS_H
#include <strings.h>
#else
#include "mystrings.h"
#endif				/* STRINGS_H */
#include "externs.h"
#include "functions.h"
/* #include "../server/server.h" */
#include "mycdef.h"
#ifdef __alpha__
#include "select.h"
#endif

/* function prototypes */
extern int _readpacket _((int, struct Cbuf *));

/* extern int errno; */
extern int restart;
static fd_set  fdr, efdr;
static int     timeisup;

int
serverserve()
{
	int     ret, i;
	long    loopcount = 0;	/* used to break loop for periodic didpoll */

	/* look at fd where new clients connect */
	if (restart == 0) {
		FD_ZERO(&fdset);
		FD_SET(port_fd, &fdset);
	} else
		for (i = 0; i < MAX_USERS + 1; i++)
			cbufs[i].new = 1;

	if (port_fd > highestfd)
		highestfd = port_fd;

	signal(SIGALRM, gotalarm);
	timeisup = 0;

	for (;;) {
		if (polldelay) {
			if (setitimer(ITIMER_REAL, polldelay, 0) < 0) {
				perror("server: setitimer failed");
				mdb("server: setitimer failed");
				exit(-1);
			}
			timeisup = 0;
/*			sigpause(0); */
		}
		bcopy((char *) &fdset, (char *) &fdr,
		    (unsigned int) sizeof(fdset));
		bcopy((char *) &fdset, (char *) &efdr,
		    (unsigned int) sizeof(fdset));

		/* wait for input */
		/* NULL should be changed to a time value that reflects */
		/* how much activity is going on */

		setpolltimeout(10000);

		if ((ret = select(highestfd + 1, &fdr, 0, &efdr, polltimeout)) < 1) {
			if (ret == 0 || errno == EINTR) {
/*				s_didpoll(); ORIG KILL */
				s_didpoll(2);
				timeisup = 0;
			} else {
				perror("select");
				return -1;
			}
		}
		if (ret > 0)
			sdoinput();
		if (timeisup || !(++loopcount % 10))
/*			s_didpoll(); ORIG KILL */
			s_didpoll(2);
	}
	return 0;
}

void
gotalarm()
{
	timeisup++;
#if defined(linux) || defined(hpux) || defined(_SEQUENT_) || defined(SOLARIS)
	signal(SIGALRM, gotalarm);
#endif	/* (linux || hpux || _SEQUENT_ || SOLARIS) */
}

void
sdoinput()
{
	int     n;
	register int x;

	for (x = 1; x <= highestfd; x++)
		if (FD_ISSET(x, &efdr) > 0) {
			disconnectuser(x);
		}
	/* examine set of file descriptors */
	for (x = 1; x <= highestfd; x++)
		if (FD_ISSET(x, &fdr) > 0) {
			if (x == port_fd) {
				/* new connect on advertised socket */
				if ((n = _newconnect(x)) > 0)
					/* let server init the user */
					s_new_user(n);
			} else {
				/* fd is client input */
				switch (_readpacket(x, &cbufs[x])) {
				case 1:
					/* complete packet */
					s_packet(x, cbufs[x].buf);
					break;
				case 0:
					/* incomplete packet */
					break;
				case -1:
					/* error */
					break;
				case -2:
					/* close connection */
					disconnectuser(x);
					break;
				}
			}
		}
}
