/* Copyright (c) 1991 by John Atwood deVries II. */
/* For copying and distribution information, see the file COPYING. */

/* signal handling routines */

#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#ifdef STRINGS_H
#include <strings.h>
#else
#include "../murgil/mystrings.h"
#endif				/* STRINGS_H */
#ifdef _SEQUENT_
#include <sys/select.h>
#endif				/* _SEQUENT_ */
#include "server.h"
#include "externs.h"

/* The signal stuff is not portable, and should be in unix.c. */

#define	mask(s)	(1 << ((s)-1))

/* set up the signal trappers */
extern int highestfd;
extern fd_set fdset;
extern int port_fd;

void
trapsignals()
{

	/* exit on a hangup or terminate signal */
	signal(SIGHUP, icbrestart);
	signal(SIGTERM, icbexit);
	signal(SIGINT, cycle_logs);
	signal(SIGUSR1, icbdump);
	signal(SIGUSR2, icbload);
}

void
icbdump()
{
	FILE   *dump;
	int     i, j, k;
	char    tbuf[255];

	if ((dump = fopen("icbd.dump", "w")) == NULL) {
		sprintf(tbuf, "[ERROR] icbd.dump open: %s", sys_errlist[errno]);
		mdb(tbuf);
		return;
	}
	for (i = 0; i < 8; i++)
		fprintf(dump, "%ld\n", fdset.fds_bits[i]);
	fprintf(dump, "%d\n", port_fd);
	fprintf(dump, "%d\n", highestfd);
	for (i = 0; i <= MAX_REAL_USERS; i++)
		fprintf(dump, "%d\n", S_kill[i]);
	for (i = 0; i < MAX_USERS; i++) {
		fprintf(dump, "%sX\n", u_tab[i].loginid);
		fprintf(dump, "%sX\n", u_tab[i].nodeid);
		fprintf(dump, "%sX\n", u_tab[i].nickname);
		fprintf(dump, "%sX\n", u_tab[i].password);
		fprintf(dump, "%sX\n", u_tab[i].realname);
		fprintf(dump, "%sX\n", u_tab[i].group);
		fprintf(dump, "%d\n", u_tab[i].login);
		fprintf(dump, "%d\n", u_tab[i].echoback);
		fprintf(dump, "%d\n", u_tab[i].t_notify);
		fprintf(dump, "%ld\n", u_tab[i].t_on);
		fprintf(dump, "%ld\n", u_tab[i].t_sent);
		fprintf(dump, "%ld\n", u_tab[i].t_recv);
		fprintf(dump, "%ld\n", u_tab[i].t_group);
		k = nlcount(*u_tab[i].pri_n_hushed);
		fprintf(dump, "%d\n", k);
		if (k > 0)
			for (j = 0; j < k; j++)
				fprintf(dump, "%sX\n", nlget(u_tab[i].pri_n_hushed));
		k = nlcount(*u_tab[i].pub_n_hushed);
		fprintf(dump, "%d\n", k);
		if (k > 0)
			for (j = 0; j < k; j++)
				fprintf(dump, "%sX\n", nlget(u_tab[i].pub_n_hushed));
		k = nlcount(*u_tab[i].pri_s_hushed);
		fprintf(dump, "%d\n", k);
		if (k > 0)
			for (j = 0; j < k; j++)
				fprintf(dump, "%sX\n", nlget(u_tab[i].pri_s_hushed));
		k = nlcount(*u_tab[i].pub_s_hushed);
		fprintf(dump, "%d\n", k);
		if (k > 0)
			for (j = 0; j < k; j++)
				fprintf(dump, "%sX\n", nlget(u_tab[i].pub_s_hushed));
		k = nlcount(*u_tab[i].n_notifies);
		fprintf(dump, "%d\n", k);
		if (k > 0)
			for (j = 0; j < k; j++)
				fprintf(dump, "%sX\n", nlget(u_tab[i].n_notifies));
		k = nlcount(*u_tab[i].s_notifies);
		fprintf(dump, "%d\n", k);
		if (k > 0)
			for (j = 0; j < k; j++)
				fprintf(dump, "%sX\n", nlget(u_tab[i].s_notifies));
	}
	for (i = 0; i < MAX_GROUPS; i++) {
		fprintf(dump, "%sX\n", g_tab[i].name);
		fprintf(dump, "%sX\n", g_tab[i].topic);
		fprintf(dump, "%d\n", g_tab[i].visibility);
		fprintf(dump, "%d\n", g_tab[i].control);
		fprintf(dump, "%d\n", g_tab[i].mod);
		fprintf(dump, "%ld\n", g_tab[i].modtimeout);
		fprintf(dump, "%sX\n", g_tab[i].missingmod);
		fprintf(dump, "%d\n", g_tab[i].volume);
		k = nlcount(*g_tab[i].n_invites);
		fprintf(dump, "%d\n", k);
		if (k > 0)
			for (j = 0; j < k; j++)
				fprintf(dump, "%sX\n", nlget(g_tab[i].n_invites));
		k = nlcount(*g_tab[i].nr_invites);
		fprintf(dump, "%d\n", k);
		if (k > 0)
			for (j = 0; j < k; j++)
				fprintf(dump, "%sX\n", nlget(g_tab[i].nr_invites));
		k = nlcount(*g_tab[i].s_invites);
		fprintf(dump, "%d\n", k);
		if (k > 0)
			for (j = 0; j < k; j++)
				fprintf(dump, "%sX\n", nlget(g_tab[i].s_invites));
		k = nlcount(*g_tab[i].sr_invites);
		fprintf(dump, "%d\n", k);
		if (k > 0)
			for (j = 0; j < k; j++)
				fprintf(dump, "%sX\n", nlget(g_tab[i].sr_invites));
		k = nlcount(*g_tab[i].n_bars);
		fprintf(dump, "%d\n", k);
		if (k > 0)
			for (j = 0; j < k; j++)
				fprintf(dump, "%sX\n", nlget(g_tab[i].n_bars));
		k = nlcount(*g_tab[i].n_nr_bars);
		fprintf(dump, "%d\n", k);
		if (k > 0)
			for (j = 0; j < k; j++)
				fprintf(dump, "%sX\n", nlget(g_tab[i].n_nr_bars));
		k = nlcount(*g_tab[i].s_bars);
		fprintf(dump, "%d\n", k);
		if (k > 0)
			for (j = 0; j < k; j++)
				fprintf(dump, "%sX\n", nlget(g_tab[i].s_bars));
		k = nlcount(*g_tab[i].s_nr_bars);
		fprintf(dump, "%d\n", k);
		if (k > 0)
			for (j = 0; j < k; j++)
				fprintf(dump, "%sX\n", nlget(g_tab[i].s_nr_bars));
	}
	fclose(dump);
}

void
icbload()
{
	FILE   *dump;
	int     i, j;
	long    k;
	char    name[80], name2[80];
	char    tbuf[255];

	if ((dump = fopen("icbd.dump", "r")) == NULL) {
		sprintf(tbuf, "[ERROR] icbd.dump open: %s", sys_errlist[errno]);
		mdb(tbuf);
		return;
	}
	init_groups();
	clear_users();
	for (i = 0; i < 8; i++) {
		fscanf(dump, "%ld\n", &k);
		fdset.fds_bits[i] = k;
	}
	fscanf(dump, "%d\n", &port_fd);
	fscanf(dump, "%d\n", &highestfd);
	for (i = 0; i <= MAX_REAL_USERS; i++) {
		fscanf(dump, "%ld\n", &k);
		S_kill[i] = k;
	}
	for (i = 0; i < MAX_USERS; i++) {
		fgets(name, 80, dump);
		strncpy(u_tab[i].loginid, name, strlen(name) - 2);
		u_tab[i].loginid[strlen(name) - 2] = '\0';
		fgets(name, 80, dump);
		strncpy(u_tab[i].nodeid, name, strlen(name) - 2);
		u_tab[i].nodeid[strlen(name) - 2] = '\0';
		fgets(name, 80, dump);
		strncpy(u_tab[i].nickname, name, strlen(name) - 2);
		u_tab[i].nickname[strlen(name) - 2] = '\0';
		fgets(name, 80, dump);
		strncpy(u_tab[i].password, name, strlen(name) - 2);
		u_tab[i].password[strlen(name) - 2] = '\0';
		fgets(name, 80, dump);
		strncpy(u_tab[i].realname, name, strlen(name) - 2);
		u_tab[i].realname[strlen(name) - 2] = '\0';
		fgets(name, 80, dump);
		strncpy(u_tab[i].group, name, strlen(name) - 2);
		u_tab[i].group[strlen(name) - 2] = '\0';
		fscanf(dump, "%d\n", &j);
		u_tab[i].login = j;
		fscanf(dump, "%d\n", &j);
		u_tab[i].echoback = j;
		fscanf(dump, "%d\n", &j);
		u_tab[i].t_notify = j;
		fscanf(dump, "%ld\n", &k);
		u_tab[i].t_on = k;
		fscanf(dump, "%ld\n", &k);
		u_tab[i].t_sent = k;
		fscanf(dump, "%ld\n", &k);
		u_tab[i].t_recv = k;
		fscanf(dump, "%ld\n", &k);
		u_tab[i].t_group = k;
		fscanf(dump, "%d\n", &j);
		for (; j > 0; j--) {
			fscanf(dump, "%d\n", &j);
			fgets(name, 80, dump);
			memset(name2, '\0', 80);
			strncpy(name2, name, strlen(name) - 2);
			nlput(u_tab[i].pri_n_hushed, name2);
		}
		fscanf(dump, "%d\n", &j);
		for (; j > 0; j--) {
			fscanf(dump, "%d\n", &j);
			fgets(name, 80, dump);
			bzero(name2, 80);
			strncpy(name2, name, strlen(name) - 2);
			nlput(u_tab[i].pub_n_hushed, name2);
		}
		fscanf(dump, "%d\n", &j);
		for (; j > 0; j--) {
			fscanf(dump, "%d\n", &j);
			fgets(name, 80, dump);
			bzero(name2, 80);
			strncpy(name2, name, strlen(name) - 2);
			nlput(u_tab[i].pri_s_hushed, name2);
		}
		fscanf(dump, "%d\n", &j);
		for (; j > 0; j--) {
			fscanf(dump, "%d\n", &j);
			fgets(name, 80, dump);
			bzero(name2, 80);
			strncpy(name2, name, strlen(name) - 2);
			nlput(u_tab[i].pub_s_hushed, name2);
		}
		fscanf(dump, "%d\n", &j);
		for (; j > 0; j--) {
			fscanf(dump, "%d\n", &j);
			fgets(name, 80, dump);
			bzero(name2, 80);
			strncpy(name2, name, strlen(name) - 2);
			nlput(u_tab[i].n_notifies, name2);
		}
		fscanf(dump, "%d\n", &j);
		for (; j > 0; j--) {
			fgets(name, 80, dump);
			bzero(name2, 80);
			strncpy(name2, name, strlen(name) - 2);
			nlput(u_tab[i].s_notifies, name2);
		}
	}
	for (i = 0; i < MAX_GROUPS; i++) {
		fgets(name, 80, dump);
		strncpy(g_tab[i].name, name, strlen(name) - 2);
		g_tab[i].name[strlen(name) - 2] = '\0';
		fgets(name, 80, dump);
		strncpy(g_tab[i].topic, name, strlen(name) - 2);
		g_tab[i].topic[strlen(name) - 2] = '\0';
		fscanf(dump, "%d\n", &j);
		g_tab[i].visibility = j;
		fscanf(dump, "%d\n", &j);
		g_tab[i].control = j;
		fscanf(dump, "%d\n", &j);
		g_tab[i].mod = j;
		fscanf(dump, "%ld\n", &k);
		g_tab[i].modtimeout = k;
		fgets(name, 80, dump);
		strncpy(g_tab[i].missingmod, name, strlen(name) - 2);
		g_tab[i].missingmod[strlen(name) - 2] = '\0';
		fscanf(dump, "%d\n", &j);
		g_tab[i].volume = j;
		fscanf(dump, "%d\n", &j);
		for (; j > 0; j--) {
			fgets(name, 80, dump);
			bzero(name2, 80);
			strncpy(name2, name, strlen(name) - 2);
			nlput(g_tab[i].n_invites, name2);
		}
		fscanf(dump, "%d\n", &j);
		for (; j > 0; j--) {
			fgets(name, 80, dump);
			bzero(name2, 80);
			strncpy(name2, name, strlen(name) - 2);
			nlput(g_tab[i].nr_invites, name2);
		}
		fscanf(dump, "%d\n", &j);
		for (; j > 0; j--) {
			fgets(name, 80, dump);
			bzero(name2, 80);
			strncpy(name2, name, strlen(name) - 2);
			nlput(g_tab[i].s_invites, name2);
		}
		fscanf(dump, "%d\n", &j);
		for (; j > 0; j--) {
			fgets(name, 80, dump);
			bzero(name2, 80);
			strncpy(name2, name, strlen(name) - 2);
			nlput(g_tab[i].sr_invites, name2);
		}
		fscanf(dump, "%d\n", &j);
		for (; j > 0; j--) {
			fgets(name, 80, dump);
			bzero(name2, 80);
			strncpy(name2, name, strlen(name) - 2);
			nlput(g_tab[i].n_bars, name2);
		}
		fscanf(dump, "%d\n", &j);
		for (; j > 0; j--) {
			fgets(name, 80, dump);
			bzero(name2, 80);
			strncpy(name2, name, strlen(name) - 2);
			nlput(g_tab[i].n_nr_bars, name2);
		}
		fscanf(dump, "%d\n", &j);
		for (; j > 0; j--) {
			fgets(name, 80, dump);
			bzero(name2, 80);
			strncpy(name2, name, strlen(name) - 2);
			nlput(g_tab[i].s_bars, name2);
		}
		fscanf(dump, "%d\n", &j);
		for (; j > 0; j--) {
			fgets(name, 80, dump);
			bzero(name2, 80);
			strncpy(name2, name, strlen(name) - 2);
			nlput(g_tab[i].s_nr_bars, name2);
		}
	}
	fclose(dump);
	unlink("icbd.dump");
}

void
icbrestart()
{
	int	user;
	char   *args[5];
	char   *env[2];
	char    tbuf[255];
	extern int port_num;

	mdb("[RESTART] Server restarting");
	icbdump();
	close_logs();
	args[0] = "icbd";
	args[1] = "-R";
	args[2] = "-p";
	sprintf(tbuf, "%d", port_num);
	args[3] = tbuf;
	args[4] = NULL;
	env[0] = "RESTART=Y";
	env[1] = NULL;
	(void) fcntl(6, F_GETFD, 0);
	if (fork())
		exit(0);
	if (execve("./icbd", args, env) < 0) {
		sprintf(tbuf, "[ERROR] RESTART failed: %s",
		    sys_errlist[errno]);
		mdb(tbuf);
		for (user = 0; user < MAX_REAL_USERS; user++)
			if (u_tab[user].login == 1)
				sendimport(user, "Restart",
				    "Restart not done.");
		mdb("[RESTART] Restart failed");
	}
}
